@@ -113,6 +113,9 @@
 #' * \code{hr_CI_type}: The type of confidence interval for hazard ratio,
 #'   either "log-rank p-value" or "bootstrap".
 #'
+#' * \code{event_summary}: A data frame containing the count and percentage
+#'   of deaths and switches by treatment arm.
+#'
 #' * \code{Sstar}: A data frame containing the counterfactual untreated
 #'   survival times and event indicators for each treatment group.
 #'   The variables include \code{id}, \code{stratum}, 
@@ -128,58 +131,28 @@
 #'   and \code{treat}.
 #' 
 #' * \code{fit_aft}: The fitted AFT model for estimating \code{psi}.
+#' 
+#' * \code{res_aft}: The deviance residuals from the fitted AFT model.
 #'
 #' * \code{data_outcome}: The input data for the outcome Cox model of 
 #'   counterfactual unswitched survival times.
 #'   The variables include \code{id}, \code{stratum}, \code{"t_star"}, 
 #'   \code{"d_star"}, \code{"treated"}, \code{base_cov}, and \code{treat}.
 #'
+#' * \code{km_outcome}: The Kaplan-Meier estimates of the survival
+#'   functions for the treatment and control groups based on the
+#'   counterfactual unswitched survival times.
+#'   
+#' * \code{lr_outcome}: The log-rank test results for the treatment
+#'   effect based on the counterfactual unswitched survival times.
+#'   
 #' * \code{fit_outcome}: The fitted outcome Cox model.
 #'
 #' * \code{fail}: Whether a model fails to converge.
 #'
 #' * \code{psimissing}: Whether the `psi` parameter cannot be estimated.
 #'
-#' * \code{settings}: A list with the following components:
-#'
-#'     - \code{aft_dist}: The distribution for time to event for the AFT
-#'       model.
-#'
-#'     - \code{strata_main_effect_only}: Whether to only include the strata
-#'       main effects in the AFT model.
-#'
-#'     - \code{low_psi}: The lower limit of the causal parameter.
-#'     
-#'     - \code{hi_psi}: The upper limit of the causal parameter.
-#'     
-#'     - \code{treat_modifier}: The sensitivity parameter for the constant
-#'       treatment effect assumption.
-#'
-#'     - \code{recensor}: Whether to apply recensoring to counterfactual
-#'       survival times.
-#'
-#'     - \code{admin_recensor_only}: Whether to apply recensoring to
-#'       administrative censoring times only.
-#'
-#'     - \code{autoswitch}: Whether to exclude recensoring for treatment 
-#'       arms with no switching.
-#'
-#'     - \code{root_finding}: The univariate root-finding algorithm to use.
-#'     
-#'     - \code{alpha}: The significance level to calculate confidence
-#'       intervals.
-#'
-#'     - \code{ties}: The method for handling ties in the Cox model.
-#'
-#'     - \code{tol}: The desired accuracy (convergence tolerance) 
-#'       for \code{psi}.
-#'
-#'     - \code{boot}: Whether to use bootstrap to obtain the confidence
-#'       interval for hazard ratio.
-#'
-#'     - \code{n_boot}: The number of bootstrap samples.
-#'
-#'     - \code{seed}: The seed to reproduce the bootstrap results.
+#' * \code{settings}: A list containing the input parameter values.
 #'
 #' * \code{fail_boots}: The indicators for failed bootstrap samples
 #'   if \code{boot} is \code{TRUE}.
@@ -219,7 +192,7 @@
 #'   rx = "rx", censor_time = "censyrs", aft_dist = "weibull",
 #'   boot = FALSE)
 #'
-#' c(fit1$hr, fit1$hr_CI)
+#' fit1
 #'
 #' # Example 2: two-way treatment switching (illustration only)
 #'
@@ -242,7 +215,7 @@
 #'                "pathway.f"),
 #'   aft_dist = "weibull", boot = FALSE)
 #'
-#' c(fit2$hr, fit2$hr_CI)
+#' fit2
 #'
 #' @export
 ipe <- function(data, id = "id", stratum = "", time = "time", 
@@ -331,5 +304,45 @@ ipe <- function(data, id = "id", stratum = "", time = "time",
     }
   }
   
+  
+  # convert treatment back to a factor variable if needed
+  if (is.factor(data[[treat]])) {
+    levs = levels(data[[treat]])
+    
+    out$event_summary[[treat]] <- factor(out$event_summary[[treat]], 
+                                         levels = c(1,2), labels = levs)
+    
+    out$Sstar[[treat]] <- factor(out$Sstar[[treat]], 
+                                 levels = c(1,2), labels = levs)
+    
+    out$kmstar[[treat]] <- factor(out$kmstar[[treat]], 
+                                  levels = c(1,2), labels = levs)
+    
+    out$data_aft[[treat]] <- factor(out$data_aft[[treat]], 
+                                    levels = c(1,2), labels = levs)
+    
+    out$data_outcome[[treat]] <- factor(out$data_outcome[[treat]], 
+                                        levels = c(1,2), labels = levs)
+    
+    out$km_outcome[[treat]] <- factor(out$km_outcome[[treat]], 
+                                      levels = c(1,2), labels = levs)
+  }
+  
+  
+  out$settings <- list(
+    data = data, id = id, stratum = stratum, time = time, 
+    event = event, treat = treat, rx = rx, 
+    censor_time = censor_time, base_cov = base_cov, 
+    aft_dist = aft_dist,
+    strata_main_effect_only = strata_main_effect_only,
+    low_psi = low_psi, hi_psi = hi_psi, 
+    treat_modifier = treat_modifier, recensor = recensor,
+    admin_recensor_only = admin_recensor_only,
+    autoswitch = autoswitch, root_finding = root_finding,
+    alpha = alpha, ties = ties, tol = tol, boot = boot,
+    n_boot = n_boot, seed = seed
+  )
+  
+  class(out) <- "ipe"
   out
 }
