@@ -97,6 +97,9 @@
 survfit_phregr <- function(object, newdata, sefit = TRUE,
                            conftype = "log-log", conflev = 0.95) {
 
+  if (!inherits(object, "phregr"))
+    stop("object must be of class 'phregr'");
+  
   p = object$p
   if (p == 0) {
     beta = 0
@@ -108,14 +111,14 @@ survfit_phregr <- function(object, newdata, sefit = TRUE,
 
   basehaz = object$basehaz
 
-  covariates = object$covariates
-  stratum = object$stratum
-  offset = object$offset
-  id = object$id
+  covariates = object$settings$covariates
+  stratum = object$settings$stratum
+  offset = object$settings$offset
+  id = object$settings$id
 
   if (id != "") {
-    tstart = object$time
-    tstop = object$time2
+    tstart = object$settings$time
+    tstop = object$settings$time2
   } else {
     tstart = ""
     tstop = ""
