@@ -389,8 +389,6 @@ List tsesimpcpp(const DataFrame data,
   }
   
   
-  // covariates for the accelerated failure time model for control with pd
-  // including swtrt, stratum and base2_cov
   int q; // number of columns corresponding to the strata effects
   if (strata_main_effect_only) {
     q = sum(d - 1);
@@ -398,6 +396,8 @@ List tsesimpcpp(const DataFrame data,
     q = nstrata - 1;
   }
   
+  // covariates for the accelerated failure time model for control with pd
+  // including swtrt, stratum and base2_cov
   StringVector covariates_aft(q+p2+1);
   NumericMatrix z_aftn(n,q+p2);
   covariates_aft[0] = "swtrt";
@@ -543,8 +543,43 @@ List tsesimpcpp(const DataFrame data,
   n = sum(sub);
   if (n == 0) stop("no observations left after removing missing values");
   
+  // summarize number of deaths and switches by treatment arm
+  IntegerVector treat_out = IntegerVector::create(0, 1);
+  NumericVector n_total(2);
+  NumericVector n_event(2);
+  NumericVector n_pd(2);
+  NumericVector n_switch(2);
+  for (int i = 0; i < n; ++i) {
+    int g = treatn[i];
+    n_total[g]++;
+    if (eventn[i] == 1) n_event[g]++;
+    if (pdn[i] == 1) n_pd[g]++;
+    if (swtrtn[i] == 1) n_switch[g]++;
+  }
   
-  DataFrame lr = lrtest(data, "", stratum, treat, time, "", event, "", 0, 0);
+  // Compute percentages
+  NumericVector pct_event(2);
+  NumericVector pct_pd(2);
+  NumericVector pct_switch(2);
+  for (int g = 0; g < 2; g++) {
+    pct_event[g] = 100.0 * n_event[g] / n_total[g];
+    pct_pd[g] = 100.0 * n_pd[g] / n_total[g];
+    pct_switch[g] = 100.0 * n_switch[g] / n_total[g];
+  }
+  
+  // Combine count and percentage
+  List event_summary = List::create(
+    _["treated"] = treat_out,
+    _["n"] = n_total,
+    _["event_n"] = n_event,
+    _["event_pct"] = pct_event,
+    _["pd_n"] = n_pd,
+    _["pd_pct"] = pct_pd,
+    _["switch_n"] = n_switch,
+    _["switch_pct"] = pct_switch
+  );
+  
+  DataFrame lr = lrtest(data, "", stratum, treat, time, "", event, "", 0,0,0);
   double logRankPValue = lr["logRankPValue"];
   double zcrit = R::qnorm(1-alpha/2, 0, 1, 1, 0);
   
@@ -573,7 +608,7 @@ List tsesimpcpp(const DataFrame data,
                   double psi1lower = NA_REAL, psi1upper = NA_REAL;
                   
                   // initialize data_aft and fit_aft
-                  List data_aft(2), fit_aft(2);
+                  List data_aft(2), fit_aft(2), res_aft(2);
                   if (k == -1) {
                     for (int h=0; h<2; ++h) {
                       List data_x = List::create(
@@ -607,6 +642,22 @@ List tsesimpcpp(const DataFrame data,
                       }
                       
                       fit_aft[h] = fit_x;
+                      
+                      List res_x = List::create(
+                        Named("res") = R_NilValue,
+                        Named(treat) = R_NilValue
+                      );
+                      
+                      if (type_treat == LGLSXP ||
+                          type_treat == INTSXP) {
+                        res_x[treat] = treatwi[1-h];
+                      } else if (type_treat == REALSXP) {
+                        res_x[treat] = treatwn[1-h];
+                      } else if (type_treat == STRSXP) {
+                        res_x[treat] = treatwc[1-h];
+                      }
+                      
+                      res_aft[h] = res_x;
                     }
                   }
                   
@@ -614,6 +665,7 @@ List tsesimpcpp(const DataFrame data,
                   List fit_outcome;
                   double hrhat = NA_REAL, hrlower = NA_REAL, 
                     hrupper = NA_REAL, pvalue = NA_REAL;
+                  List km_outcome, lr_outcome;
                   
                   bool psimissing = false;
                   
@@ -663,6 +715,15 @@ List tsesimpcpp(const DataFrame data,
                       psiupper = -beta1[1] + zcrit*sebeta1[1];
                     }
                     
+                    NumericVector res;
+                    if (k == -1) {
+                      NumericMatrix vbeta1(q+p2+1, q+p2+1);
+                      NumericMatrix rr = residuals_liferegcpp(
+                        beta1, vbeta1, data1, "", "pps", "", "event",
+                        covariates_aft, "", "", "", dist, "deviance", 0, 0);
+                      res = rr(_,0);
+                    }
+                    
                     // update treatment-specific causal parameter estimates
                     if (h == 0) {
                       psi0hat = psihat;
@@ -678,6 +739,50 @@ List tsesimpcpp(const DataFrame data,
                       }
                     }
                     
+                    // update data_aft, fit_aft, and res_aft
+                    if (k == -1) {
+                      IntegerVector stratum2 = stratumb[l];
+                      
+                      if (has_stratum) {
+                        for (int i=0; i<p_stratum; ++i) {
+                          std::string s = as<std::string>(stratum[i]);
+                          SEXP col_stratum = u_stratum[s];
+                          SEXPTYPE type_stratum = TYPEOF(col_stratum);
+                          if (type_stratum == INTSXP) {
+                            IntegerVector v = col_stratum;
+                            data1.push_back(v[stratum2], s);
+                          } else if (type_stratum == REALSXP) {
+                            NumericVector v = col_stratum;
+                            data1.push_back(v[stratum2], s);
+                          } else if (type_stratum == STRSXP) {
+                            StringVector v = col_stratum;
+                            data1.push_back(v[stratum2], s);
+                          }
+                        }
+                      }
+                      
+                      if (type_id == INTSXP) {
+                        data1.push_front(idwi[id2], id);
+                      } else if (type_id == REALSXP) {
+                        data1.push_front(idwn[id2], id);
+                      } else if (type_id == STRSXP) {
+                        data1.push_front(idwc[id2], id);
+                      }
+                      
+                      List data_x = data_aft[h];
+                      data_x["data"] = as<DataFrame>(data1);
+                      data_aft[h] = data_x;
+                      
+                      List fit_x = fit_aft[h];
+                      fit_x["fit"] = fit1;
+                      fit_aft[h] = fit_x;
+                      
+                      List res_x = res_aft[h];
+                      res_x["res"] = res;
+                      res_aft[h] = res_x;
+                    }
+                    
+                    
                     if (!std::isnan(psihat)) {
                       // calculate counter-factual survival times
                       double a = exp(psihat);
@@ -702,50 +807,12 @@ List tsesimpcpp(const DataFrame data,
                           }
                         }
                       }
-                      
-                      // update data_aft and fit_aft
-                      if (k == -1) {
-                        IntegerVector stratum2 = stratumb[l];
-                        
-                        if (has_stratum) {
-                          for (int i=0; i<p_stratum; ++i) {
-                            std::string s = as<std::string>(stratum[i]);
-                            SEXP col_stratum = u_stratum[s];
-                            SEXPTYPE type_stratum = TYPEOF(col_stratum);
-                            if (type_stratum == INTSXP) {
-                              IntegerVector v = col_stratum;
-                              data1.push_back(v[stratum2], s);
-                            } else if (type_stratum == REALSXP) {
-                              NumericVector v = col_stratum;
-                              data1.push_back(v[stratum2], s);
-                            } else if (type_stratum == STRSXP) {
-                              StringVector v = col_stratum;
-                              data1.push_back(v[stratum2], s);
-                            }
-                          }
-                        }
-                        
-                        if (type_id == INTSXP) {
-                          data1.push_front(idwi[id2], id);
-                        } else if (type_id == REALSXP) {
-                          data1.push_front(idwn[id2], id);
-                        } else if (type_id == STRSXP) {
-                          data1.push_front(idwc[id2], id);
-                        }
-                        
-                        List data_x = data_aft[h];
-                        data_x["data"] = as<DataFrame>(data1);
-                        data_aft[h] = data_x;
-                        
-                        List fit_x = fit_aft[h];
-                        fit_x["fit"] = fit1;
-                        fit_aft[h] = fit_x;
-                      }
                     } else {
                       psimissing = 1;
                     }
                   }
                   
+                  
                   if (!psimissing) {
                     // Cox model for hypothetical treatment effect estimate
                     data_outcome = List::create(
@@ -762,6 +829,18 @@ List tsesimpcpp(const DataFrame data,
                       data_outcome.push_back(u, zj);
                     }
                     
+                    // generate KM estimate and log-rank test
+                    if (k == -1) {
+                      km_outcome = kmest(
+                        data_outcome, "", "treated", "t_star", "", 
+                        "d_star", "", "log-log", 1-alpha, 1);
+                      
+                      lr_outcome = lrtest(
+                        data_outcome, "", "ustratum", "treated", "t_star", 
+                        "", "d_star", "", 0, 0, 0);
+                    }
+                    
+                    // fit the outcome model
                     fit_outcome = phregcpp(
                       data_outcome, "", "ustratum", "t_star", "", "d_star",
                       covariates, "", "", "", ties, init, 
@@ -786,7 +865,10 @@ List tsesimpcpp(const DataFrame data,
                     out = List::create(
                       Named("data_aft") = data_aft,
                       Named("fit_aft") = fit_aft,
+                      Named("res_aft") = res_aft,
                       Named("data_outcome") = data_outcome,
+                      Named("km_outcome") = km_outcome,
+                      Named("lr_outcome") = lr_outcome,
                       Named("fit_outcome") = fit_outcome,
                       Named("psihat") = psi0hat,
                       Named("psilower") = psi0lower,
@@ -820,7 +902,10 @@ List tsesimpcpp(const DataFrame data,
   
   List data_aft = out["data_aft"];
   List fit_aft = out["fit_aft"];
+  List res_aft = out["res_aft"];
   List data_outcome = out["data_outcome"];
+  List km_outcome = out["km_outcome"];
+  List lr_outcome = out["lr_outcome"];
   List fit_outcome = out["fit_outcome"];
   
   double psihat = out["psihat"];
@@ -843,6 +928,23 @@ List tsesimpcpp(const DataFrame data,
   String hr_CI_type;
   
   if (!psimissing) {
+    // summarize number of deaths by treatment arm in the outcome data
+    IntegerVector treated = data_outcome["treated"];
+    IntegerVector event_out = data_outcome["d_star"];
+    NumericVector n_event_out(2);
+    for (int i = 0; i < n; ++i) {
+      int g = treated[i];
+      if (event_out[i] == 1) n_event_out[g]++;
+    }
+    
+    NumericVector pct_event_out(2);
+    for (int g = 0; g < 2; g++) {
+      pct_event_out[g] = 100.0 * n_event_out[g] / n_total[g];
+    }
+    
+    event_summary.push_back(n_event_out, "event_out_n");
+    event_summary.push_back(pct_event_out, "event_out_pct");
+    
     IntegerVector uid = data_outcome["uid"];
     if (type_id == INTSXP) {
       data_outcome.push_front(idwi[uid], id);
@@ -852,7 +954,16 @@ List tsesimpcpp(const DataFrame data,
       data_outcome.push_front(idwc[uid], id);
     }
     
-    IntegerVector treated = data_outcome["treated"];
+    treated = event_summary["treated"];
+    if (type_treat == LGLSXP || type_treat == INTSXP) {
+      event_summary.push_back(treatwi[1-treated], treat);
+    } else if (type_treat == REALSXP) {
+      event_summary.push_back(treatwn[1-treated], treat);
+    } else if (type_treat == STRSXP) {
+      event_summary.push_back(treatwc[1-treated], treat);
+    }
+    
+    treated = data_outcome["treated"];
     if (type_treat == LGLSXP || type_treat == INTSXP) {
       data_outcome.push_back(treatwi[1-treated], treat);
     } else if (type_treat == REALSXP) {
@@ -861,6 +972,15 @@ List tsesimpcpp(const DataFrame data,
       data_outcome.push_back(treatwc[1-treated], treat);
     }
     
+    treated = km_outcome["treated"];
+    if (type_treat == LGLSXP || type_treat == INTSXP) {
+      km_outcome.push_back(treatwi[1-treated], treat);
+    } else if (type_treat == REALSXP) {
+      km_outcome.push_back(treatwn[1-treated], treat);
+    } else if (type_treat == STRSXP) {
+      km_outcome.push_back(treatwc[1-treated], treat);
+    }
+    
     if (has_stratum) {
       IntegerVector ustratum = data_outcome["ustratum"];
       for (int i=0; i<p_stratum; ++i) {
@@ -1057,7 +1177,8 @@ List tsesimpcpp(const DataFrame data,
       double loghr = log(hrhat);
       LogicalVector ok = (!fails) & !is_na(hrhats);
       int n_ok = sum(ok);
-      NumericVector loghrs = log(hrhats[ok]);
+      NumericVector subset_hrhats = hrhats[ok];
+      NumericVector loghrs = log(subset_hrhats);
       double sdloghr = sd(loghrs);
       double tcrit = R::qt(1-alpha/2, n_ok-1, 1, 0);
       hrlower = exp(loghr - tcrit*sdloghr);
@@ -1079,19 +1200,6 @@ List tsesimpcpp(const DataFrame data,
     }
   }
   
-  List settings = List::create(
-    Named("aft_dist") = aft_dist,
-    Named("strata_main_effect_only") = strata_main_effect_only,
-    Named("recensor") = recensor,
-    Named("admin_recensor_only") = admin_recensor_only,
-    Named("swtrt_control_only") = swtrt_control_only,
-    Named("alpha") = alpha,
-    Named("ties") = ties,
-    Named("offset") = offset,
-    Named("boot") = boot,
-    Named("n_boot") = n_boot,
-    Named("seed") = seed);
-  
   List result = List::create(
     Named("psi") = psihat,
     Named("psi_CI") = NumericVector::create(psilower, psiupper),
@@ -1101,13 +1209,16 @@ List tsesimpcpp(const DataFrame data,
     Named("hr") = hrhat,
     Named("hr_CI") = NumericVector::create(hrlower, hrupper),
     Named("hr_CI_type") = hr_CI_type,
+    Named("event_summary") = as<DataFrame>(event_summary),
     Named("data_aft") = data_aft,
     Named("fit_aft") = fit_aft,
+    Named("res_aft") = res_aft,
     Named("data_outcome") = as<DataFrame>(data_outcome),
+    Named("km_outcome") = as<DataFrame>(km_outcome),
+    Named("lr_outcome") = as<DataFrame>(lr_outcome),
     Named("fit_outcome") = fit_outcome,
     Named("fail") = fail,
-    Named("psimissing") = psimissing,
-    Named("settings") = settings);
+    Named("psimissing") = psimissing);
   
   if (!swtrt_control_only) {
     result.push_back(psi1hat, "psi_trt");
